import { Alert } from "react-native";
import { is2Wflow } from "react-native-digitcamera/src/common/Constants";
import { getActivityid } from "react-native-digitcamera/src/common/Services/getActivityid";
import UploadFile from "react-native-digitcamera/src/Screens/UploadFile";
import { async } from "validate.js";
import Database from "../../../../../Components/common/DAsyncStorage";
import store from "react-native-digitcamera/src/redux/store";
import { NamedModal } from "react-native-digitcamera/src/common/GlobalModal";
import {
  callChassisPoseDetection,
  callPoseDetection,
  canCreateActivityId,
  getCurrentActivity,
  updateActivityId,
} from "react-native-digitcamera/src/Screens/OfflineFlow/CommonFunc";
import { startOfflineBackgroundUpload } from "react-native-digitcamera/src/Screens/SilentUpload/SilentUploadUtils";
import { initMediaFile } from "react-native-digitcamera/src/Screens/SilentUploadV2/DatabaseHelpers";
import { UELogEvent } from "react-native-digitcamera/src/common/libEventLog/UserExperior/LibUserExperior";
import {
  errorRecord,
  firebaseAttributes,
} from "react-native-digitcamera/src/common/libEventLog/libCrashlytics/libCrashlyticsEvent";
var sensorImageDetails = [];

export const CreateActivityId = async (
  activityDetails,
  updateReduxCallback,
  addSensorDataCallback
) => {
  var vechicleData = await Database.get("PIVechicleData");

  const AppLoginStatus = await Database.get("AppLoginStatus");
  try {
    console.log("**** CreateActivityId called");
    if (
      canCreateActivityId(activityDetails?.activityId) ||
      activityDetails?.activityId == undefined ||
      activityDetails?.activityId == null
    ) {
      //API call
      console.log("In if     ");
      const { getUserDetails } = require("../../common/Who");
      await getUserDetails();
      const {
        emailId,
        name,
        phoneNo,
        panHolderName,
        panNo,
        agentNumber,
      } = require("../../common/Who");
      const USERTYPE = AppLoginStatus?.userType;
      let body;
      var registrationNumber = activityDetails?.regNumber;
      var quoteNumber = activityDetails?.policyNumber;
      var activityTypeId = activityDetails?.activityTypeId;
      var toTutorial = vechicleData?.toTutorial;
      var toWithQuote = vechicleData?.toWithQuote;

      if (USERTYPE === "Customer") {
        body = {
          activityTypeId: activityTypeId,
          customerDetailData: {
            emailId: emailId,
            name: name,
            phoneNo: phoneNo,
          },
          licensePlateNumber: registrationNumber,
          previousPolicyExpired: true,
          quoteNumber: quoteNumber,
        };
      } else if (USERTYPE === "DealerOrAgent") {
        body = {
          activityTypeId: activityTypeId,
          customerDetailData: {
            emailId: emailId,
            name: name,
            phoneNo: phoneNo,
            panHolderName: panHolderName,
            panNo: panNo,
          },
          agentId: agentNumber,
          licensePlateNumber: registrationNumber?.value,
          previousPolicyExpired: true,
          quoteNumber: quoteNumber,
        };
      }

      var activityData = await getActivityid({ body });
      await updateActivityId(activityDetails, activityData?.activityID);

      /**
       * If getActivityId failed
       * then show failure modal
       */
      if (!activityData) {
        updateActivityKey(activityDetails.activityId, "serviceFailure", true);
        await Database.set("PIReUploadFailed", "true");
        return;
      }

      if (activityData.flag === "Manual Inspection") {
        store.dispatch({
          type: "GLOBAL_MODAL_SHOW",
          payload: NamedModal.MANUAL_INSPECTION,
        });
        return;
      }

      registrationNumber = activityData?.additionalActivitiesList.filter(
        (item) => {
          if (item.key == "Registration Number") {
            return item;
          }
        }
      );
      let userNCB = activityData?.additionalActivitiesList.filter((item) => {
        if (item.key == "User NCB") {
          return item;
        }
      });

      // update activityid in PIMediaStatus database
      console.log(
        "updateActivityId activityDetails : ",
        activityDetails,
        activityData?.activityID
      );

      console.log(
        "is2Wflow(activityData.activityTypeId",
        is2Wflow(activityData.activityTypeId),
        activityData.activityTypeId
      );

      // only update current activity data in PIVechicleData
      if (is2Wflow(activityData.activityTypeId)) {
        try {
          await Database.update("PIVechicleData", {
            activityId: activityData?.activityID,
            activityTypeId: activityData?.activityTypeId,
            licensePlateNumber:
              registrationNumber === undefined
                ? ""
                : registrationNumber[0]?.value,
            toWithQuote: toWithQuote,
            toTutorial: toTutorial,
            quoteNumber: quoteNumber,
            ncb:
              userNCB === undefined || userNCB.length == 0
                ? 0
                : Number(userNCB[0].value),
            vechicleType: "",
          });
          var activity = await getCurrentActivity();
          await checkPoseDetection(activity);
        } catch (error) {
          console.log("Error:  is2Wflow(activityData.activityTypeId) ", error);
        }
      }

      const PIVechicleData = await Database.get("PIVechicleData");
      console.log(
        "PIVechicleData : in CreateActivity.js : after creating new Activity ",
        PIVechicleData
      );

      var log_PIVechicleData = await Database.get("PIVechicleData");
      console.log(
        "PIVechicleData CreateActivityId.js ",
        JSON.stringify(log_PIVechicleData, null, "  ")
      );
      updateReduxCallback();
    }
  } catch (error) {
    console.log("Error in CreateActivityId.js ", error);
  }
};

// this function adds "value" agains a "key"  to a Activity in PIMediaStatus DB

// I'm using this to update if any of the service fails
//  /preinpsection & /instruction if failed
// then serviceFailure = true
const updateActivityKey = async (activityId, key, value) => {
  const pMediaStatus = await Database.get("PIMediaStatus");
  const nPIMediaStatus = pMediaStatus.map((item) => {
    if (item.activityId === activityId) {
      const newObj = item;
      newObj[key] = value;
      return newObj;
    } else {
      return item;
    }
  });
  Database.set("PIMediaStatus", nPIMediaStatus);
};

const addSensorImageData = async (ext, ImageStatus, docName) => {
  const PILocation = await Database.get("PILocation");
  const PIRetryCount = await Database.get("PIRetryCount");
  let offlineImageData = await Database.get("PIOfflineVechicleData");
  var currentLatitude = PILocation?.lat;
  var currentLongitude = PILocation?.long;
  var activityId = offlineImageData?.activityId;
  let imageName = docName;
  var attemptCount = PIRetryCount?.retryCount + 1;
  var fileName = activityId + "_" + imageName + ext;
  //adding sensor data
  sensorImageDetails.push({
    attemptCount: attemptCount,
    fileName: fileName,
    latitude: currentLatitude,
    longitude: currentLongitude,
    pose: imageName,
    success: ImageStatus,
    timestamp: new Date().getTime(),
  });
};

const checkPoseDetection = async (activity) => {
  UELogEvent("Check Pose Detection started in InvalidImages");
  try {
    var invalidData = activity.mediaUpload;
    for (var item of invalidData) {
      console.log(
        "invalidImages.js checkPoseDetection imageName item?.imageName ",
        item?.imageName?.includes("RCB")
      );
      if (
        item?.imageName == "SF" ||
        item?.imageName == "VDO" ||
        item?.imageName == "SF" ||
        item?.imageName == "CNG" ||
        item?.imageName == "IWS" ||
        item?.imageName?.includes("RCB") ||
        item?.imageName == "PUC"
      ) {
        console.log("item?.imageUri item?.imageUri ", item?.uri, item);
        var ext = imageName == "VDO" ? ".mp4" : ",jpg";
        await initMediaFile(item?.uri, item?.imageName, ext, "uploading");
      } else if (item?.imageName === "CHP") {
        try {
          await callChassisPoseDetection(
            item?.uri,
            item?.imageName,
            async () => {
              await chassiesPoseDetectionSucess(item?.uri, item?.imageName);
            },
            async () => {
              await chassiesPDetectionFailed(item?.uri, item?.imageName);
            }
          );
        } catch (error) {
          await chassiesPDetectionFailed(item?.uri, item?.imageName);
          console.log("Error : callChassisPoseDetection call : ", error);
        }
      } else {
        try {
          await callPoseDetection(
            item?.uri,
            item?.imageName,
            async () => {
              await poseDetectionSucess(item?.uri, item?.imageName);
            },
            async () => {
              await poseDetectionFailed(item?.uri, item?.imageName);
            }
          );
        } catch (error) {
          await poseDetectionFailed(item?.uri, item?.imageName);
          console.log("Error : callPoseDetection call : ", error);
        }
      }
    }

    /**
     * After generating activityId for uploading documents
     * whatever failed documents are there, all them had activityId 0
     * so following block of code maps the generated activity id to such items
     */
    const PIVechicleData = await Database.get("PIVechicleData");
    const mediaStatusArr = await Database.get("PIMediaStatus");

    const newMediaStatusArr = await mediaStatusArr.map((item) => {
      if (item?.activityId === 0) {
        return {
          ...item,
          activityId: PIVechicleData?.activityId,
        };
      }
      return item;
    });
    // await Database.remove("PIMediaStatus");
    await Database.set("PIMediaStatus", newMediaStatusArr);
    UELogEvent("Check Pose Detection Stopped in try");
  } catch (error) {
    UELogEvent("Check Pose Detection Stopped in Catch", {
      error: JSON.stringify(error),
    });
    firebaseAttributes(error);
    errorRecord(error);
    console.log("in checkPoseDetection() ", error);
  }
};

const chassiesPoseDetectionSucess = async (uri, imageName) => {
  try {
    console.log("createActivity : chassiesPoseDetectionSucess ", imageName);
    console.log("Uploading chassiesPoseDetectionSucess ", imageName);
  } catch (error) {
    console.log("Error : chassiesPoseDetectionSucess : ", error);
  }
};

const chassiesPDetectionFailed = async (uri, imageName) => {
  try {
    console.log("createActivity : chassiesPDetectionFailed ", imageName);
    var ext = imageName == "VDO" ? ".mp4" : ",jpg";
    await initMediaFile(uri, imageName, ext, "poseFailed");
  } catch (error) {
    console.log("Error : chassiesPDetectionFailed : ", error);
  }
};

const poseDetectionSucess = async (uri, imageName) => {
  try {
    console.log("createActivity : poseDetectionSucess ", imageName);
    console.log("Uploading poseDetectionSucess ", imageName);
    // await uploadImage(uri, imageName);
  } catch (error) {
    console.log("Error : poseDetectionSucess : ", error);
  }
};

const poseDetectionFailed = async (uri, imageName) => {
  try {
    console.log("createActivity : poseDetectionFailed ", imageName);
    var ext = imageName == "VDO" ? ".mp4" : ",jpg";
    await initMediaFile(uri, imageName, ext, "poseFailed");
  } catch (error) {
    console.log("Error : poseDetectionFailed : ", error);
  }
};
