import {
  Alert,
  Linking,
  PermissionsAndroid,
  ToastAndroid,
  Platform,
} from "react-native";
import React, { useCallback, useEffect, useRef, useState } from "react";
import Geolocation from "react-native-geolocation-service";

import Database from "../../../../Components/common/DAsyncStorage";
import { render } from "@testing-library/react-native";
import { View } from "react-native";

import { UELogEvent } from "../common/libEventLog/UserExperior/LibUserExperior";
import { useFocusEffect } from "@react-navigation/core";
import {
  getLocationPermission,
  onLocationPermissionBlocked,
  LOCATION_PERMISSION_RESULTS,
} from "./Permissions";
import { NamedModal } from "react-native-digitcamera/src/common/GlobalModal";
import store from "react-native-digitcamera/src/redux/store";

const updateLocation = async (position) => {
  console.log("LocationHandler . updateLocation . position - ", position);
  await Database.update("PILocation", {
    lat: position?.coords?.latitude.toString().substring(0, 10),
    long: position?.coords?.longitude.toString().substring(0, 10),
  });
};

let permissionRequestCount = 1;

export default function GetLocation() {
  const watchId = useRef(null);

  useFocusEffect(
    useCallback(() => {
      watchLocation();
      return () => {
        if (watchId.current !== null) {
          Geolocation.clearWatch(watchId.current);
          watchId.current = null;
        }
      };
    }, [])
  );

  const watchLocation = async () => {
    await Database.update("PILocation", {
      lat: "NA",
      long: "NA",
    });
    let hasPermission = await getLocationPermission();
    if (hasPermission === LOCATION_PERMISSION_RESULTS.DENIED) {
      store.dispatch({
        type: "LOCATION_DENIED",
      });
      Alert.alert(
        "Location required!",
        "To smoothen your process we recommend you to enable device location.",
        [
          {
            text: "Ok",
            onPress: () => {
              console.log(
                "Permission requested count  ",
                permissionRequestCount
              );
              if (permissionRequestCount < 2) {
                watchLocation();
                permissionRequestCount++;
              } else {
                store.dispatch({
                  type: "LOCATION_BLOCKED",
                });
                store.dispatch({
                  type: "GLOBAL_MODAL_SHOW",
                  payload: NamedModal.OPEN_APP_SETTINGS,
                });
              }
            },
          },
        ]
      );
      return;
    } else if (hasPermission === LOCATION_PERMISSION_RESULTS.GRANTED) {
      store.dispatch({
        type: "LOCATION_GRANTED",
      });
      const options = {
        accuracy: {
          android: "balanced",
          ios: "best",
        },
        enableHighAccuracy: true,
        timeout: 2000,
        maximumAge: 10000,
        distanceFilter: 50,
        forceRequestLocation: true,
        forceLocationManager: true,
        showLocationDialog: true,
      };
      watchId.current = Geolocation.watchPosition(
        (position) => {
          //get lat and long
          const longitude = JSON.stringify(position.coords.longitude);
          const latitude = JSON.stringify(position.coords.latitude);

          //if undefined
          if (longitude === undefined && latitude === undefined) {
            UELogEvent("Location Updates", {
              Position: JSON.stringify(position),
            });
          }
          updateLocation(position);
        },
        (error) => {
          UELogEvent("Geolocation.watchPosition error", {
            Position: JSON.stringify(error),
          });
          console.log("Location Error ", error.message);
        },
        options
      );

      Geolocation.getCurrentPosition(
        (position) => {
          updateLocation(position);
        },
        (error) => {
          UELogEvent("Geolocation.watchPosition error", {
            Position: JSON.stringify(error),
          });
          console.log("Location Error ", error.message);
        },
        options
      );
      return;
    } else if (hasPermission === LOCATION_PERMISSION_RESULTS.BLOCKED) {
      store.dispatch({
        type: "LOCATION_BLOCKED",
      });
      store.dispatch({
        type: "GLOBAL_MODAL_SHOW",
        payload: NamedModal.OPEN_APP_SETTINGS,
      });
    }
  };

  return null; // render nothing
}
