import {
  PermissionsAndroid,
  Alert,
  Linking,
  Platform,
  ToastAndroid,
} from "react-native";
import {
  UELogEvent,
  UELogMessage,
} from "./libEventLog/UserExperior/LibUserExperior";

import Geolocation from "react-native-geolocation-service";
import store from "react-native-digitcamera/src/redux/store";

export const getAndroidPermission = async (onGranted, onRejected) => {
  console.log("*** Permission getAndroidPermission before");
  const result = await PermissionsAndroid.requestMultiple([
    PermissionsAndroid.PERMISSIONS.CAMERA,
    PermissionsAndroid.PERMISSIONS.RECORD_AUDIO,
    PermissionsAndroid.PERMISSIONS.WRITE_EXTERNAL_STORAGE,
    PermissionsAndroid.PERMISSIONS.READ_EXTERNAL_STORAGE,
    PermissionsAndroid.PERMISSIONS.READ_PHONE_STATE,
  ]);
  console.log("*** Permission getAndroidPermission after await ", result);
  if (
    result["android.permission.CAMERA"] === "granted" &&
    result["android.permission.RECORD_AUDIO"] === "granted" &&
    result["android.permission.WRITE_EXTERNAL_STORAGE"] === "granted" &&
    result["android.permission.READ_EXTERNAL_STORAGE"] === "granted" &&
    result["android.permission.READ_PHONE_STATE"] === "granted"
  ) {
    console.log("*** Permission getAndroidPermission granted all");
    onGranted();
    UELogEvent("Permission.js, Permissions. ", {
      "External Storage Write": JSON.stringify("granted"),
      "External Storage Read": JSON.stringify("granted"),
      "Phone State": JSON.stringify("granted"),
      Camera: JSON.stringify("granted"),
      Audio: JSON.stringify("granted"),
    });
  } else if (
    result["android.permission.CAMERA"] === "never_ask_again" ||
    result["android.permission.RECORD_AUDIO"] === "never_ask_again" ||
    result["android.permission.WRITE_EXTERNAL_STORAGE"] === "never_ask_again" ||
    result["android.permission.READ_EXTERNAL_STORAGE"] === "never_ask_again" ||
    result["android.permission.READ_PHONE_STATE"] === "never_ask_again"
  ) {
    console.log("*** Permission getAndroidPermission denied any one");
    Alert.alert(
      translate("Permission denied"),
      "We will be not able to proceed your vehicle inspection"
    );
    onRejected();
  }
};

export const LOCATION_PERMISSION_RESULTS = {
  GRANTED: "GRANTED",
  DENIED: "DENIED", // deny [android | ios]
  BLOCKED: "BLOCKED", //never ask again [android] | deny [ios]
};

export const onLocationPermissionDenied = () => {};

export const detectPermissionStatus = async () => {
  const result = await checkPermissions();
  if (result) {
    store.dispatch({
      type: "GLOBAL_MODAL_HIDE",
    });
    store.dispatch({
      type: "LOCATION_GRANTED",
    });
  } else {
    openAppSettings();
  }
};

const openAppSettings = () => {
  if (Platform.OS === "ios") {
    Linking.canOpenURL("app-settings:")
      .then((supported) => {
        if (!supported) {
          Alert.alert("Unable to open settings");
        } else {
          return Linking.openURL("app-settings:");
        }
      })
      .catch((err) => {
        Alert.alert("Unable to open settings");
      });
  } else {
    try {
      Linking.openSettings();
    } catch (e) {
      console.log("Error in LocationHandler.js . openSettings -> ", e);
    }
  }
};

const checkPermissions = async () => {
  if (Platform.OS === "android") {
    const result = await PermissionsAndroid.check(
      PermissionsAndroid.PERMISSIONS.ACCESS_FINE_LOCATION
    );
    if (result) {
      return LOCATION_PERMISSION_RESULTS.GRANTED;
    }
    return result;
  }
};

export const getLocationPermission = async () => {
  if (Platform.OS === "ios") {
    const result = await requestLocationPermissionIOS();
    return result;
  }
  if (Platform.OS === "android") {
    if (Platform.Version < 23) {
      return LOCATION_PERMISSION_RESULTS.GRANTED;
    }
    const result = await PermissionsAndroid.check(
      PermissionsAndroid.PERMISSIONS.ACCESS_FINE_LOCATION
    );

    if (result) {
      return LOCATION_PERMISSION_RESULTS.GRANTED;
    }

    const status = await PermissionsAndroid.request(
      PermissionsAndroid.PERMISSIONS.ACCESS_FINE_LOCATION
    );

    if (status === PermissionsAndroid.RESULTS.GRANTED) {
      return LOCATION_PERMISSION_RESULTS.GRANTED;
    }

    if (status === PermissionsAndroid.RESULTS.DENIED) {
      ToastAndroid.show(
        "Location permission denied. Can not proceed.",
        ToastAndroid.LONG
      );
      return LOCATION_PERMISSION_RESULTS.DENIED;
    } else if (status === PermissionsAndroid.RESULTS.NEVER_ASK_AGAIN) {
      return LOCATION_PERMISSION_RESULTS.BLOCKED;
    }
  }

  return LOCATION_PERMISSION_RESULTS.DENIED;
};

const requestLocationPermissionIOS = async () => {
  const status = await Geolocation.requestAuthorization("whenInUse");
  if (status === "granted") {
    return LOCATION_PERMISSION_RESULTS.GRANTED;
  }
  if (status === "denied") {
    return LOCATION_PERMISSION_RESULTS.BLOCKED;
  }
  return LOCATION_PERMISSION_RESULTS.BLOCKED;
};
